@@ -660,6 +660,8 @@ class SeriesEdit(EditWithComplete, ToMetadataMixin):
 
     def initialize(self, db, id_):
         self.books_to_refresh = set()
+        if 'series' in db.new_api.pref('categories_using_hierarchy', default=()):
+            self.set_hierarchy_separator('.')
         self.update_items_cache(db.new_api.all_field_names('series'))
         series = db.new_api.field_for('series', id_)
         self.current_val = self.original_val = series or ''
@@ -1230,6 +1232,8 @@ class Cover(ImageView):  # {{{
         m = super().build_context_menu()
         m.addSeparator()
         m.addAction(QIcon.ic('view-image'), _('View image in popup window'), self.view_image)
+        from calibre.gui2.book_details import create_open_cover_with_menu
+        create_open_cover_with_menu(self, m, _('Edit cover with...'))
         return m
 
     def mouseDoubleClickEvent(self, event):
@@ -1247,6 +1251,32 @@ class Cover(ImageView):  # {{{
             from calibre.utils.img import image_to_data
             self.current_val = image_to_data(d.current_img.toImage(), fmt='png')
 
+    def open_with(self, entry):
+        from calibre.gui2 import info_dialog
+        from calibre.gui2.open_with import run_program
+        from calibre.utils.img import image_from_data, save_image
+        cdata = self.current_val
+        img = image_from_data(cdata)
+        pt = PersistentTemporaryFile(suffix='.png')
+        pt.close()
+        try:
+            save_image(img, pt.name)
+            run_program(entry, pt.name, self)
+            info_dialog(self, _('Cover opened in {}').format(entry.get('name') or _('external editor')), _(
+                'Close this popup when you are done making changes to the cover.'), show=True, show_copy_button=False)
+        finally:
+            with open(pt.name, 'rb') as f:
+                ncdata = f.read()
+            os.remove(pt.name)
+            if ncdata and ncdata != cdata:
+                self.current_val = ncdata
+
+    def choose_open_with(self):
+        from calibre.gui2.open_with import choose_program
+        entry = choose_program('cover_image', self)
+        if entry is not None:
+            self.open_with(entry)
+
     def undo_trim(self):
         if self.cdata_before_trim:
             self.current_val = self.cdata_before_trim
@@ -1502,6 +1532,8 @@ class TagsEdit(EditWithComplete, ToMetadataMixin):  # {{{
 
     def initialize(self, db, id_):
         self.books_to_refresh = set()
+        if 'tags' in db.new_api.pref('categories_using_hierarchy', default=()):
+            self.set_hierarchy_separator('.')
         tags = db.tags(id_, index_is_id=True)
         tags = tags.split(',') if tags else []
         self.current_val = tags
