@@ -72,15 +72,10 @@ def initialize_calibre():
     if hasattr(initialize_calibre, 'initialized'):
         return
     initialize_calibre.initialized = True
-
     # Ensure that all temp files/dirs are created under a calibre tmp dir
-    from calibre.ptempfile import base_dir
-    try:
-        base_dir()
-    except OSError:
-        pass  # Ignore this error during startup, so we can show a better error message to the user later.
+    from calibre.ptempfile import fix_tempfile_module
+    fix_tempfile_module()
 
-    #
     # Ensure that the max number of open files is at least 1024
     if iswindows:
         # See https://msdn.microsoft.com/en-us/library/6e3b887c.aspx
@@ -107,6 +102,16 @@ def initialize_calibre():
         return get_debug_executable() + ['--fix-multiprocessing', '--', prog]
     spawn.get_command_line = get_command_line
     orig_spawn_passfds = util.spawnv_passfds
+    orig_remove_temp_dir = util._remove_temp_dir
+
+    def safe_rmtree(rmtree):
+        def r(tdir):
+            if tdir and os.path.exists(tdir):
+                rmtree(tdir)
+        return r
+
+    def safe_remove_temp_dir(rmtree, tdir):
+        orig_remove_temp_dir(safe_rmtree(rmtree), tdir)
 
     def wrapped_orig_spawn_fds(args, passfds):
         # as of python 3.11 util.spawnv_passfds expects bytes args
@@ -122,6 +127,7 @@ def initialize_calibre():
         patched_args = get_debug_executable() + ['--fix-multiprocessing', '--'] + args[idx + 1:]
         return wrapped_orig_spawn_fds(patched_args, passfds)
     util.spawnv_passfds = spawnv_passfds
+    util._remove_temp_dir = safe_remove_temp_dir
 
     #
     # Setup resources
