@@ -3,7 +3,7 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 '''
 Defines various abstract base classes that can be subclassed to create powerful news fetching recipes.
 '''
-__docformat__ = "restructuredtext en"
+__docformat__ = 'restructuredtext en'
 
 
 import io
@@ -38,8 +38,7 @@ from polyglot.builtins import string_or_bytes
 
 def classes(classes):
     q = frozenset(classes.split(' '))
-    return dict(attrs={
-        'class': lambda x: x and frozenset(x.split()).intersection(q)})
+    return dict(attrs={'class': lambda x: x and frozenset(x.split()).intersection(q)})
 
 
 def prefixed_classes(classes):
@@ -243,7 +242,7 @@ class BasicNewsRecipe(Recipe):
     #:
     #:    {
     #:     name      : 'tag name',   #e.g. 'div'
-    #:     attrs     : a dictionary, #e.g. {'class': 'advertisment'}
+    #:     attrs     : a dictionary, #e.g. {'class': 'advertisement'}
     #:    }
     #:
     #: All keys are optional. For a full explanation of the search criteria, see
@@ -366,11 +365,12 @@ class BasicNewsRecipe(Recipe):
     ignore_duplicate_articles = None
 
     # The following parameters control how the recipe attempts to minimize
-    # JPEG image sizes
+    # image sizes. Note that if compression is enabled PNG images are converted
+    # to JPEG.
 
     #: Set this to False to ignore all scaling and compression parameters and
     #: pass images through unmodified. If True and the other compression
-    #: parameters are left at their default values, JPEG images will be scaled to fit
+    #: parameters are left at their default values, images will be scaled to fit
     #: in the screen dimensions set by the output profile and compressed to size at
     #: most (w * h)/16 where w x h are the scaled image dimensions.
     compress_news_images = False
@@ -623,7 +623,7 @@ class BasicNewsRecipe(Recipe):
         for key in article.keys():
             if key.endswith('_origlink'):
                 url = article[key]
-                if url and (url.startswith('http://') or url.startswith('https://')):
+                if url and (url.startswith(('http://', 'https://'))):
                     return url
         ans = article.get('link', None)
         if not ans and getattr(article, 'links', None):
@@ -645,7 +645,7 @@ class BasicNewsRecipe(Recipe):
         `soup`: A `BeautifulSoup <https://www.crummy.com/software/BeautifulSoup/bs4/doc/>`__
         instance containing the downloaded :term:`HTML`.
         '''
-        return None
+        return
 
     def abort_article(self, msg=None):
         ''' Call this method inside any of the preprocess methods to abort the
@@ -672,7 +672,7 @@ class BasicNewsRecipe(Recipe):
             try:
                 raw_html = self.extract_readable_article(raw_html, url)
             except:
-                self.log.exception('Auto cleanup of URL: %r failed'%url)
+                self.log.exception(f'Auto cleanup of URL: {url!r} failed')
 
         return raw_html
 
@@ -724,7 +724,7 @@ class BasicNewsRecipe(Recipe):
         try:
             parts = urlparse(url)
         except Exception:
-            self.log.error('Failed to parse url: %r, ignoring' % url)
+            self.log.error(f'Failed to parse url: {url!r}, ignoring')
             return frozenset()
         nl = parts.netloc
         path = parts.path or ''
@@ -752,7 +752,7 @@ class BasicNewsRecipe(Recipe):
             with closing(open_func(url_or_raw, timeout=self.timeout)) as f:
                 _raw = f.read()
             if not _raw:
-                raise RuntimeError('Could not fetch index from %s'%url_or_raw)
+                raise RuntimeError(f'Could not fetch index from {url_or_raw}')
         else:
             _raw = url_or_raw
         if raw:
@@ -800,13 +800,11 @@ class BasicNewsRecipe(Recipe):
             root = frag
         elif frag.tag == 'body':
             root = document_fromstring(
-                '<html><head><title>%s</title></head></html>' %
-                extracted_title)
+                f'<html><head><title>{extracted_title}</title></head></html>')
             root.append(frag)
         else:
             root = document_fromstring(
-                '<html><head><title>%s</title></head><body/></html>' %
-                extracted_title)
+                f'<html><head><title>{extracted_title}</title></head><body/></html>')
             root.xpath('//body')[0].append(frag)
 
         body = root.xpath('//body')[0]
@@ -911,7 +909,7 @@ class BasicNewsRecipe(Recipe):
 
         src = src.replace('\\', '/')
         if re.search(r'feed_\d+/article_\d+/images/img', src, flags=re.I) is None:
-            self.log.warn('Ignoring invalid TOC thumbnail image: %r'%src)
+            self.log.warn(f'Ignoring invalid TOC thumbnail image: {src!r}')
             return
         article.toc_thumbnail = re.sub(r'^.*?feed', 'feed',
                 src, flags=re.IGNORECASE)
@@ -1108,7 +1106,6 @@ class BasicNewsRecipe(Recipe):
                 article = self.feed_objects[f].articles[a]
             except:
                 self.log.exception('Failed to get article object for postprocessing')
-                pass
             else:
                 self.populate_article_metadata(article, ans, first_fetch)
         return ans
@@ -1158,7 +1155,7 @@ class BasicNewsRecipe(Recipe):
         templ = templ(lang=self.lang_for_html)
         css = self.template_css + '\n\n' +(self.get_extra_css() or '')
         timefmt = self.timefmt
-        return templ.generate(self.title, "mastheadImage.jpg", timefmt, feeds,
+        return templ.generate(self.title, 'mastheadImage.jpg', timefmt, feeds,
                               extra_css=css).render(doctype='xhtml')
 
     @classmethod
@@ -1182,7 +1179,7 @@ class BasicNewsRecipe(Recipe):
             from calibre.utils.cleantext import clean_xml_chars
 
             # Truncating the string could cause a dangling UTF-16 half-surrogate, which will cause lxml to barf, clean it
-            ans = clean_xml_chars(ans) + '\u2026'
+            ans = clean_xml_chars(ans) + '…'
         return ans
 
     def feed2index(self, f, feeds):
@@ -1199,7 +1196,7 @@ class BasicNewsRecipe(Recipe):
                 if bn:
                     bn = bn.rpartition('/')[-1]
                     if bn:
-                        img = os.path.join(imgdir, 'feed_image_%d%s'%(self.image_counter, os.path.splitext(bn)[-1]))
+                        img = os.path.join(imgdir, f'feed_image_{self.image_counter}{os.path.splitext(bn)[-1]}')
                         try:
                             with open(img, 'wb') as fi, closing(self.browser.open(feed.image_url, timeout=self.timeout)) as r:
                                 fi.write(r.read())
@@ -1288,8 +1285,7 @@ class BasicNewsRecipe(Recipe):
                             seen.add(val)
 
         for feed, article in remove:
-            self.log.debug('Removing duplicate article: %s from section: %s'%(
-                article.title, feed.title))
+            self.log.debug(f'Removing duplicate article: {article.title} from section: {feed.title}')
             feed.remove_article(article)
 
         if self.remove_empty_feeds:
@@ -1340,14 +1336,14 @@ class BasicNewsRecipe(Recipe):
 
         self.feed_objects = feeds
         for f, feed in enumerate(feeds):
-            feed_dir = os.path.join(self.output_dir, 'feed_%d'%f)
+            feed_dir = os.path.join(self.output_dir, f'feed_{f}')
             if not os.path.isdir(feed_dir):
                 os.makedirs(feed_dir)
 
             for a, article in enumerate(feed):
                 if a >= self.max_articles_per_feed:
                     break
-                art_dir = os.path.join(feed_dir, 'article_%d'%a)
+                art_dir = os.path.join(feed_dir, f'article_{a}')
                 if not os.path.isdir(art_dir):
                     os.makedirs(art_dir)
                 try:
@@ -1388,8 +1384,8 @@ class BasicNewsRecipe(Recipe):
                 break
 
         for f, feed in enumerate(feeds):
-            html = self.feed2index(f,feeds)
-            feed_dir = os.path.join(self.output_dir, 'feed_%d'%f)
+            html = self.feed2index(f, feeds)
+            feed_dir = os.path.join(self.output_dir, f'feed_{f}')
             with open(os.path.join(feed_dir, 'index.html'), 'wb') as fi:
                 fi.write(html)
         self.create_opf(feeds)
@@ -1477,7 +1473,7 @@ class BasicNewsRecipe(Recipe):
         try:
             self._download_masthead(url)
         except:
-            self.log.exception("Failed to download supplied masthead_url")
+            self.log.exception('Failed to download supplied masthead_url')
 
     def resolve_masthead(self):
         self.masthead_path = None
@@ -1492,7 +1488,7 @@ class BasicNewsRecipe(Recipe):
             # Failure sets self.masthead_path to None
             self.download_masthead(murl)
         if self.masthead_path is None:
-            self.log.info("Synthesizing mastheadImage")
+            self.log.info('Synthesizing mastheadImage')
             self.masthead_path = os.path.join(self.output_dir, 'mastheadImage.jpg')
             try:
                 self.default_masthead_image(self.masthead_path)
@@ -1587,7 +1583,7 @@ class BasicNewsRecipe(Recipe):
             ref.title = 'Masthead Image'
             opf.guide.append(ref)
 
-        manifest = [os.path.join(dir, 'feed_%d'%i) for i in range(len(feeds))]
+        manifest = [os.path.join(dir, f'feed_{i}') for i in range(len(feeds))]
         manifest.append(os.path.join(dir, 'index.html'))
         manifest.append(os.path.join(dir, 'index.ncx'))
 
@@ -1596,7 +1592,7 @@ class BasicNewsRecipe(Recipe):
         if cpath is None:
             pf = open(os.path.join(dir, 'cover.jpg'), 'wb')
             if self.default_cover(pf):
-                cpath =  pf.name
+                cpath = pf.name
         if cpath is not None and os.access(cpath, os.R_OK):
             opf.cover = cpath
             manifest.append(cpath)
@@ -1624,7 +1620,7 @@ class BasicNewsRecipe(Recipe):
             f = feeds[num]
             for j, a in enumerate(f):
                 if getattr(a, 'downloaded', False):
-                    adir = 'feed_%d/article_%d/'%(num, j)
+                    adir = f'feed_{num}/article_{j}/'
                     auth = a.author
                     if not auth:
                         auth = None
@@ -1634,12 +1630,12 @@ class BasicNewsRecipe(Recipe):
                     else:
                         desc = self.description_limiter(desc)
                     tt = a.toc_thumbnail if a.toc_thumbnail else None
-                    entries.append('%sindex.html'%adir)
+                    entries.append(f'{adir}index.html')
                     po = self.play_order_map.get(entries[-1], None)
                     if po is None:
                         self.play_order_counter += 1
                         po = self.play_order_counter
-                    arelpath = '%sindex.html'%adir
+                    arelpath = f'{adir}index.html'
                     for curl in self.canonicalize_internal_url(a.orig_url, is_link=False):
                         aumap[curl].add(arelpath)
                     article_toc_entry = parent.add_item(arelpath, None,
@@ -1655,7 +1651,7 @@ class BasicNewsRecipe(Recipe):
                                 arelpath, entry['anchor'], entry['title'] or _('Unknown section'),
                                 play_order=po
                             )
-                    last = os.path.join(self.output_dir, ('%sindex.html'%adir).replace('/', os.sep))
+                    last = os.path.join(self.output_dir, (f'{adir}index.html').replace('/', os.sep))
                     for sp in a.sub_pages:
                         prefix = os.path.commonprefix([opf_path, sp])
                         relp = sp[len(prefix):]
@@ -1682,7 +1678,7 @@ class BasicNewsRecipe(Recipe):
 
         if len(feeds) > 1:
             for i, f in enumerate(feeds):
-                entries.append('feed_%d/index.html'%i)
+                entries.append(f'feed_{i}/index.html')
                 po = self.play_order_map.get(entries[-1], None)
                 if po is None:
                     self.play_order_counter += 1
@@ -1693,11 +1689,11 @@ class BasicNewsRecipe(Recipe):
                 desc = getattr(f, 'description', None)
                 if not desc:
                     desc = None
-                feed_index(i, toc.add_item('feed_%d/index.html'%i, None,
+                feed_index(i, toc.add_item(f'feed_{i}/index.html', None,
                     f.title, play_order=po, description=desc, author=auth))
 
         else:
-            entries.append('feed_%d/index.html'%0)
+            entries.append('feed_0/index.html')
             feed_index(0, toc)
 
         for i, p in enumerate(entries):
@@ -1719,7 +1715,7 @@ class BasicNewsRecipe(Recipe):
         article = request.article
         self.log.debug('Downloaded article:', article.title, 'from', article.url)
         article.orig_url = article.url
-        article.url = 'article_%d/index.html'%a
+        article.url = f'article_{a}/index.html'
         article.downloaded = True
         article.sub_pages  = result[1][1:]
         self.jobs_done += 1
@@ -1730,7 +1726,7 @@ class BasicNewsRecipe(Recipe):
 
     def error_in_article_download(self, request, traceback):
         self.jobs_done += 1
-        if traceback and re.search('^AbortArticle:', traceback, flags=re.M) is not None:
+        if traceback and re.search(r'^AbortArticle:', traceback, flags=re.M) is not None:
             self.log.warn('Aborted download of article:', request.article.title,
                           'from', request.article.url)
             self.report_progress(float(self.jobs_done)/len(self.jobs),
@@ -1764,7 +1760,7 @@ class BasicNewsRecipe(Recipe):
                 url = url.decode('utf-8')
             if url.startswith('feed://'):
                 url = 'http'+url[4:]
-            self.report_progress(0, _('Fetching feed')+' %s...'%(title if title else url))
+            self.report_progress(0, _('Fetching feed')+f' {title if title else url}...')
             try:
                 purl = urlparse(url, allow_fragments=False)
                 if purl.username or purl.password:
@@ -1784,7 +1780,7 @@ class BasicNewsRecipe(Recipe):
                 ))
             except Exception as err:
                 feed = Feed()
-                msg = 'Failed feed: %s'%(title if title else url)
+                msg = f'Failed feed: {title if title else url}'
                 feed.populate_from_preparsed_feed(msg, [])
                 feed.description = as_unicode(err)
                 parsed_feeds.append(feed)
@@ -1861,7 +1857,7 @@ class BasicNewsRecipe(Recipe):
             item.extract()
             divtag = soup.new_tag('div')
             brtag  = soup.new_tag('br')
-            oldParent.insert(myIndex,divtag)
+            oldParent.insert(myIndex, divtag)
             divtag.append(item)
             divtag.append(brtag)
         return soup
@@ -1952,8 +1948,7 @@ class CalibrePeriodical(BasicNewsRecipe):
         self.log('Fetching downloaded recipe')
         try:
             raw = self.browser.open_novisit(
-                'https://news.calibre-ebook.com/subscribed_files/%s/0/temp.downloaded_recipe'
-                % self.calibre_periodicals_slug
+                f'https://news.calibre-ebook.com/subscribed_files/{self.calibre_periodicals_slug}/0/temp.downloaded_recipe'
                     ).read()
         except Exception as e:
             if hasattr(e, 'getcode') and e.getcode() == 403:
